# 2 "map.cppo.mli"
type key = string
# 8
type 'a t
(** The type of maps from type [key] to type ['a]. *)

val empty: 'a t
val isEmpty: 'a t -> bool
val has:  'a t -> key -> bool    

val cmpU:  'a t -> 'a t -> ('a -> 'a -> int [@bs]) -> int
val cmp:  'a t -> 'a t -> ('a -> 'a -> int) -> int

val eqU: 'a t -> 'a t -> ('a -> 'a -> bool [@bs]) -> bool
val eq: 'a t -> 'a t -> ('a -> 'a -> bool) -> bool
(** [equal m1 m2 cmp] tests whether the maps [m1] and [m2] are
   equal, that is, contain equal keys and associate them with
   equal data.  [cmp] is the equality predicate used to compare
   the data associated with the keys. *)

val forEachU: 'a t -> (key -> 'a -> unit [@bs]) ->  unit
val forEach: 'a t -> (key -> 'a -> unit) ->  unit
(** [forEach m f] applies [f] to all bindings in map [m].
   [f] receives the key as first argument, and the associated value
   as second argument.  The bindings are passed to [f] in increasing
   order with respect to the ordering over the type of the keys. *)

val reduceU:  'a t -> 'b -> ('b -> key -> 'a -> 'b [@bs]) -> 'b
val reduce:  'a t -> 'b -> ('b -> key -> 'a -> 'b) -> 'b
(** [reduce m a f] computes [(f kN dN ... (f k1 d1 a)...)],
   where [k1 ... kN] are the keys of all bindings in [m]
   (in increasing order), and [d1 ... dN] are the associated data. *)

val everyU:  'a t -> (key -> 'a -> bool [@bs]) -> bool
val every:  'a t -> (key -> 'a -> bool) -> bool
(** [every m p] checks if all the bindings of the map
    satisfy the predicate [p].
 *)

val someU:  'a t -> (key -> 'a -> bool [@bs]) -> bool
val some:  'a t -> (key -> 'a -> bool) -> bool
(** [some m p] checks if at least one binding of the map
    satisfy the predicate [p].
 *)
val size: 'a t -> int
val toList: 'a t -> (key * 'a) list
(** In increasing order with respect *)
val toArray: 'a t -> (key * 'a) array
val ofArray: (key * 'a) array -> 'a t     
val keysToArray: 'a t -> key array 
val valuesToArray: 'a t -> 'a array
val minKey: _ t -> key option 
val minKeyUndefined: _ t -> key Js.undefined
val maxKey: _ t -> key option
val maxKeyUndefined: _ t -> key Js.undefined
val minimum: 'a t -> (key * 'a) option
val minUndefined: 'a t -> (key * 'a) Js.undefined
val maximum: 'a t -> (key * 'a) option
val maxUndefined: 'a t -> (key * 'a) Js.undefined
val get: 'a t -> key -> 'a option
val getUndefined: 'a t -> key -> 'a Js.undefined
val getWithDefault:  'a t -> key -> 'a  -> 'a
val getExn: 'a t -> key -> 'a 

val checkInvariantInternal: _ t -> unit
(**
   {b raise} when invariant is not held
*)  

(****************************************************************************)

val remove: 'a t ->  key -> 'a t
(** [remove m x] returns a map containing the same bindings as
   [m], except for [x] which is unbound in the returned map. *)
val removeMany: 'a t -> key array -> 'a t

val set: 'a t ->  key -> 'a -> 'a t
(** [add m x y] returns a map containing the same bindings as
   [m], plus a binding of [x] to [y]. If [x] was already bound
   in [m], its previous binding disappears. *)

val updateU: 'a t -> key -> ('a option -> 'a option [@bs]) -> 'a t 
val update: 'a t -> key -> ('a option -> 'a option) -> 'a t 
val mergeArray: 'a t -> (key * 'a) array -> 'a t
    
val mergeU:
    'a t -> 'b t ->
    (key -> 'a option -> 'b option -> 'c option [@bs]) ->
    'c t
val merge:
    'a t -> 'b t ->
    (key -> 'a option -> 'b option -> 'c option) ->
    'c t      
(** [merge m1 m2 f] computes a map whose keys is a subset of keys of [m1]
    and of [m2]. The presence of each such binding, and the corresponding
    value, is determined with the function [f].
 *)

val keepU: 
    'a t -> 
    (key -> 'a -> bool [@bs]) -> 
    'a t
val keep: 
    'a t -> 
    (key -> 'a -> bool) -> 
    'a t      
(** [keep m p] returns the map with all the bindings in [m]
    that satisfy predicate [p].
*)

val partitionU: 
    'a t -> 
    (key -> 'a -> bool [@bs]) -> 
    'a t * 'a t
val partition: 
    'a t -> 
    (key -> 'a -> bool) -> 
    'a t * 'a t      
(** [partition m p] returns a pair of maps [(m1, m2)], where
    [m1] contains all the bindings of [s] that satisfy the
    predicate [p], and [m2] is the map with all the bindings of
    [s] that do not satisfy [p].
 *)





val split: key -> 'a t -> 'a t * 'a option * 'a t
(** [split x m] returns a triple [(l, data, r)], where
      [l] is the map with all the bindings of [m] whose key
    is strictly less than [x];
      [r] is the map with all the bindings of [m] whose key
    is strictly greater than [x];
      [data] is [None] if [m] contains no binding for [x],
      or [Some v] if [m] binds [v] to [x].
 *)


val mapU: 'a t -> ('a -> 'b [@bs]) ->  'b t
val map: 'a t -> ('a -> 'b) ->  'b t    
(** [map m f] returns a map with same domain as [m], where the
   associated value [a] of all bindings of [m] has been
   replaced by the result of the application of [f] to [a].
   The bindings are passed to [f] in increasing order
   with respect to the ordering over the type of the keys. *)

val mapWithKeyU: 'a t -> (key -> 'a -> 'b [@bs]) -> 'b t
val mapWithKey: 'a t -> (key -> 'a -> 'b) -> 'b t    

(**/**)
val checkInvariantInternal: _ t -> unit
(**
   {b raise} when invariant is not held
*)  
(**/**)

